/*
 * stepper.c
 *
 * Created: 2015-11-07 02:03:22
 *  Author: DS
 */ 



#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"
#include "ui.h"
#include "lcd.h"
#include "wdog.h"
#include "ee_mem.h"
#include "stepper.h"


const char steps[] = {
	0b11100111,		//1
	0b11101011,		//2
	0b11101011,		//3
	0b11011011,		//4
	0b11011011,		//5
	0b11010111,		//6
	0b11010111,		//7
	0b11100111		//8
};

//rise fall
//---- ----
const char swim[] = {
	0b11100011,		//1
	0b11100011,		//2
	0b11001011,		//3
	0b11001011,		//4
	0b11010011,		//5
	0b11010011,		//6
	0b11000111,		//7
	0b11000111		//8
};

#define MOTOR_OFF		0b11000011

struct {
	volatile char ticks;
	volatile char flag;
	char step;
	char pwm;
	char ocr;
	
	char next;
	uint16_t position;
	uint16_t positionmax;
	uint16_t position_index;

	char step_index;
	char pwm_index;
} motor;


//--------------------------------------------
void StepperInit(void)
{
	MOTOR_PORT =  MOTOR_OFF;
	MOTOR_DDR |= (1<<MOTOR_A_PIN | 1<<MOTOR_B_PIN | 1<<MOTOR_C_PIN | 1<<MOTOR_D_PIN);

	MOTOR_EN_SET;
	MOTOR_EN_DDR |= (1<<MOTOR_EN_PIN);
	
	TCCR1A = (1<<WGM11);
	TCCR1B = (1<<WGM13 | 1<<WGM12);
	ICR1 = PWM_PERIOD;
	OCR1A = 0;

	TIMSK1 = (1<<OCIE1A | 1<<TOIE1);
	sei();
}

//--------------------------------------------
void StepperStart(void)
{
	char step;
	char pwm;
	char ocr;
	char step_index;
	char pwm_index;

				motor.position = 0;
				motor.position_index = 0;
				motor.positionmax = (uint16_t)(UiSettingsGet(MENU_ROT) * STEPPER_MIKRO_STEPS * 2);
				step_index = 0;
				pwm_index = 0;
				
				if ((step_index & 1)) {
					ocr = (pwm_index * STEPPER_MIKRO_MULTI);
					} else {
					ocr = ((STEPPER_MIKRO_STEPS - pwm_index) * STEPPER_MIKRO_MULTI);
					//if (ocr >= PWM_PERIOD) ocr = 0;
				}

				step = steps[step_index];
				pwm = swim[step_index];
				if (ocr == 0)  step = pwm;
				
	motor.step = step;
	motor.pwm = pwm;
	motor.ocr = ocr;
	
	motor.next = 0;
	motor.ticks = 0;
	motor.flag = 0;
	
	MOTOR_PORT =  MOTOR_OFF;
	MOTOR_EN_SET;
	TCNT1 = 0;
	OCR1A = motor.ocr;
	TCCR1B = (1<<WGM13 | 1<<WGM12 | 1<<CS11 | 0<<CS10);
	sei();
	_delay_ms(100);
}

//--------------------------------------------
void StepperStop(void)
{
	TCCR1B = (1<<WGM13 | 1<<WGM12);
	MOTOR_PORT =  MOTOR_OFF;
	MOTOR_EN_OFF;
}

//--------------------------------------------
char StepperIndex(char action)
{
	if (action == 1) motor.position_index++;
	return motor.position_index;
}

//--------------------------------------------
char StepperNext(char action)
{
	char step;
	char pwm;
	char ocr;
	char step_index;
	char pwm_index;
	uint16_t position;
	uint32_t position_temp;
	char result = STEPPER_RUN;
	
	if (motor.flag) {
		motor.flag = 0;
		
		motor.next++;
		if (motor.next >= UiSettingsGet(MENU_SPD)) {
			motor.next = 0;
						
			position_temp =  (uint32_t)motor.position_index;
			position_temp = (position_temp + 1);
			position_temp = (position_temp * UiSettingsGet(MENU_DEG));
			position_temp = (position_temp * motor.positionmax);
			position_temp = (position_temp / 360);			
			
			if (motor.position < position_temp) {
				motor.position++;
				//----------------------------------------------
				if (motor.position <= motor.positionmax) {

					if (UiSettingsGet(MENU_DIR) == 0) {
						position = motor.position;
					} else {
						position = (motor.positionmax - motor.position);
					}
				
						step_index = ((position / STEPPER_MIKRO_STEPS) % STEPPER_STEPS);
						pwm_index = (position % STEPPER_MIKRO_STEPS);
				
						if ((step_index & 1)) {
							ocr = (pwm_index * STEPPER_MIKRO_MULTI);
						} else {
							ocr = ((STEPPER_MIKRO_STEPS - pwm_index) * STEPPER_MIKRO_MULTI);
						}

					step = steps[step_index];
					pwm = swim[step_index];
					if (ocr == 0)  step = pwm;
				
					cli();
					motor.step = step;
					motor.pwm = pwm;
					motor.ocr = ocr;
					sei();
		
				} else {
				//----------------------------------------------
					result = STEPPER_STOP;
				}
			} else {
				result = STEPPER_PAUSE;
			}
		}
	}
	return result;
}

//--------------------------------------------
ISR (TIMER1_OVF_vect)
{
	MOTOR_PORT =  motor.step;
	OCR1A = motor.ocr;
	motor.ticks++;
	if (motor.ticks >= 100) {
		motor.ticks = 0;
		motor.flag = 1;
	}
}

//--------------------------------------------
ISR (TIMER1_COMPA_vect)
{
	MOTOR_PORT =  motor.pwm;
}

