/*
 * ui.c
 *
 * Created: 2015-11-05 13:41:57
 *  Author: dsosnowski
 */ 


#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "defs.h"
#include "ui.h"
#include "lcd.h"
#include "wdog.h"
#include "ee_mem.h"





//--------------------------------------------------------
struct {
	char press;
	char level;
	char param;
	char enter;
	char refresh;
} ui;


struct {
	uint16_t degree;	
	uint16_t rot;
	char dir;
	char delay;
	char speed;
	char man;
}sett;

//--------------------------------------------------------
void UiInit(void)
{
	BUTTONS_PORT |= (1<<BUTTON1_PIN | 1<<BUTTON2_PIN | 1<<BUTTON3_PIN | 1<<BUTTON4_PIN);
	
	BUZZ_OFF;
	BUZZ_DDR |= (1<<BUZZ_PIN);
	
	TRIG_OFF;
	TRIG_PORT |= (1<<TRIG_POW_PIN);
	TRIG_DDR |= (1<<TRIG_PIN | 1<<TRIG_POW_PIN);
}

//--------------------------------------------------------
char UiScan(void)
{
	char in;
	char result;
	static char prev;
	static uint16_t hold;
	
	in = BUTTONS_IN;
	result = 0;
	
	//any state
	if ((in & (1<<BUTTON1_PIN | 1<<BUTTON2_PIN | 1<<BUTTON3_PIN | 1<<BUTTON4_PIN)) == (1<<BUTTON1_PIN | 1<<BUTTON2_PIN | 1<<BUTTON3_PIN | 1<<BUTTON4_PIN))
	{
		//sw_free
		if ((hold > 3) && (hold < SWITCH_THRESHOLD)) result = prev;
		prev = 0;			
		hold = 0;
	} else {
		//if pressed
		if ((in & (1<<BUTTON4_PIN)) == 0)	result = SW_MENU;
		if ((in & (1<<BUTTON3_PIN)) == 0)	result = SW_PLUS;
		if ((in & (1<<BUTTON2_PIN)) == 0)	result = SW_MINUS;
		if ((in & (1<<BUTTON1_PIN)) == 0)	result = SW_OK;

		if (hold < 0xFFFF) hold++;			//counting hold time
		if (result != prev) hold = 0;		//new key - reset counters
		
		prev = result;
		result = 0;

		if (hold == SWITCH_THRESHOLD) {
			if ((prev == SW_PLUS) || (prev == SW_MINUS)) {	
				result =  prev;
				hold = (SWITCH_THRESHOLD - SWITCH_REPEAT);
			} else {
				result = (prev + SWITCH_HOLD);
			}
		}
	}
	ui.press = result;
	return result;
}

//--------------------------------------------------------
char UiKbdGet(void)
{
	return ui.press;
}


//--------------------------------------------------------
void UiNavi(void)
{
	if (ui.press == SW_MENU)
	{
		ui.level++;
		if (ui.level > MENU_LAST) ui.level = 0;
	}
	
	if ((ui.press == SW_PLUS) || (ui.press == SW_MINUS)) {
		UiSettingsChange(ui.level, ui.press);
	}
}

//--------------------------------------------------------
void UiMenu(void)
{
	if (ui.level == MENU_DEG) {
		LCDMsg("Obrot:",0,0);
		LCDValue4(sett.degree); LCDData(0); LCDText("     ");
		
		LCDMsg("Zdjecia:",1,0);
		LCDValue4(360 / sett.degree); LCDText("    ");
	}
	if (ui.level == MENU_DIR) {
		LCDMsg("Kierunek:",0,0);
		if (sett.dir == 0) LCDText(" Prawo "); 
		else LCDText(" Lewo  ");
		LCDMsg("                ", 1,0);
	}
	if (ui.level == MENU_DELAY) {
		LCDMsg("Przerwa:",0,0);
		LCDValue4(sett.delay); LCDText("    "); 
		LCDMsg("                ", 1,0);
	}
	if (ui.level == MENU_SPD) {
		LCDMsg("Szybkosc:",0,0);
		LCDValue4(sett.speed); LCDText("   ");
		LCDMsg("                ", 1,0);
	}
	if (ui.level == MENU_ROT) {
		LCDMsg("Kr/Obr:",0,0);
		LCDValue4(sett.rot); LCDText("     ");
		LCDMsg("                ", 1,0);
	}
	if (ui.level == MENU_MAN) {
		if (sett.man == 0) LCDMsg("Automatycznie   ",0,0);
		else LCDMsg("Manualnie       ",0,0);
		LCDMsg("                ", 1,0);
	}
}

//--------------------------------------------------------
void UiSettingsChange(char level, char action)
{
	//-------------------------------------------------
	if (level == MENU_DEG) {
		if (action == SW_PLUS) {
			if (sett.degree < 360) sett.degree++;
			while ((360 % sett.degree) > 0) {
				sett.degree++;
			} 
		}
		
		if (action == SW_MINUS) {
			if (sett.degree > 1) sett.degree--;
			while ((360 % sett.degree) > 0) {
				sett.degree--;
			}
		}
		if (sett.degree > 360) sett.degree = 10;
		if (sett.degree < 1) sett.degree = 90;
	}
	//-------------------------------------------------
	if (level == MENU_DIR) {
		if (action == SW_PLUS) sett.dir = 1;
		if (action == SW_MINUS) sett.dir = 0;				
		if (sett.dir > 1) sett.dir = 0;
	}
	//-------------------------------------------------
	if (level == MENU_DELAY) {
		if (action == SW_PLUS) {
			if (sett.delay < 20) sett.delay++;
		}
		if (action == SW_MINUS) {
			if (sett.delay > 1)  sett.delay--;
		}
		if (sett.delay > 20) sett.delay = 1;
		if (sett.delay < 1) sett.delay = 5;
		
	}
	//-------------------------------------------------
	if (level == MENU_SPD) {
		if (action == SW_PLUS) {
			if (sett.speed < 100) sett.speed++;
		}
		if (action == SW_MINUS) {
			if (sett.speed > 1)  sett.speed--;
		}
		if (sett.speed > 100) sett.speed = 10;
		if (sett.speed < 1) sett.speed = 1;
	}
	//-------------------------------------------------
	if (level == MENU_ROT) {
		if (action == SW_PLUS) {
			if (sett.rot < 250) sett.rot++;
		}
		if (action == SW_MINUS) {
			if (sett.rot > 1)  sett.rot--;
		}
		if (sett.rot > 500) sett.rot = 200;
		if (sett.rot < 1) sett.rot = 200;
	}
	//-------------------------------------------------
	if (level == MENU_MAN) {
		if (action == SW_PLUS) sett.man = 1;
		if (action == SW_MINUS) sett.man = 0;
		if (sett.man > 1) sett.man = 0;
	}
	
}

//--------------------------------------------------------
void UiSettingsDefault(void)
{
	char i;
	for(i=0; i<MENU_LAST; i++) {
		UiSettingsChange(i, 0);
	}
}

//--------------------------------------------------------
uint16_t UiSettingsGet(char index)
{
	if (index == 0) return (uint16_t)sett.degree;
	if (index == 1) return (uint16_t)sett.dir;
	if (index == 2) return (uint16_t)sett.delay;
	if (index == 3) return (uint16_t)sett.speed;
	if (index == 4) return (uint16_t)sett.rot;
	if (index == 5) return (uint16_t)sett.man;
}
